/****************************************************************************
**
** Copyright (C) 2008 Andrey Rijov <ANDron142@yandex.ru>
**
** This file is part of AQEMU.
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor,
** Boston, MA  02110-1301, USA.
**
****************************************************************************/

#include <QApplication>
#include <QMessageBox>
#include <QTranslator>
#include <QFileDialog>
#include <QDir>
#include <QFile>

#include "Utils.h"
#include "Main_Window.h"
#include "Settings_Window.h"

int main( int argc, char *argv[] )
{
	QCoreApplication::setOrganizationName( "ANDronSoft" );
	QCoreApplication::setApplicationName( "AQEMU" );
	QSettings settings;
	
	QApplication app( argc, argv );
	
	// This is Upgrade AQEMU? Find Previous Confing
	if( QFile::exists(QDir::homePath() + "/.config/ANDronSoft/AQEMU.conf") )
	{
		if( settings.value( "AQEMU_Config_Version", "0.4" ).toString() == "0.5" )
		{
			// OK. Config Version 0.5 Averable
			AQDebug( "int main( int argc, char *argv[] )",
					 "AQEMU Config Version: 0.5" );
		}
		else
		{
			// Remove Old Config!
			if( QFile::copy(QDir::homePath() + "/.config/ANDronSoft/AQEMU.conf",
				QDir::homePath() + "/.config/ANDronSoft/AQEMU.conf.bak") )
			{
				AQWarning( "int main( int argc, char *argv[] )",
						   "AQEMU Configuration File No Version 0.5. File Saved: AQEMU.conf.bak" );
				
				settings.clear();
				settings.sync();
			}
			else
			{
				AQError( "int main( int argc, char *argv[] )",
						 "Cannot Save Old Version AQEMU Configuration File!" );
			}
		}
	}
	
	settings.setValue( "AQEMU_Config_Version", "0.5" );
	
	// Use Log
	if( settings.value( "Use_Log", "yes" ).toString() == "yes" )
	{
		AQUse_Log( true );
	}
	else
	{
		AQUse_Log( false );
	}
	
	// Log File Name
	AQLog_Path( settings.value("Log_Path", "").toString() );
	
	// Check_Version_on_Start
	if( settings.value("Check_Version_on_Start", "yes").toString() == "yes" )
	{
		if( ! System_Info::Check_QEMU_Version() )
		{
			AQError( "int main( int argc, char *argv[] )",
					 "Cannot Check QEMU Version! Using Default: QEMU 0.9.0" );
			settings.setValue( "QEMU_Version", "0.9.0" );
		}
	}
	
	if( settings.value("QEMU_Version", "0.9.0").toString() == "Old" )
	{
		AQGraphic_Error( "int main( int argc, char *argv[] )", QObject::tr("Error!"),
						 QObject::tr("Installed on This System Version QEMU Cannot be Used!\n"
						 "AQEMU Working on QEMU 0.9.0 and Highter!"), true );
	}
	
	// Find Data Folder
	if( settings.value("AQEMU_Data_Folder", "").toString().isEmpty() )
	{
		#ifdef Q_OS_WIN32
		if( QDir::currentPath() )
		{
			settings.setValue( "AQEMU_Data_Folder", QDir::currentPath() );
			AQDebug( "int main( int argc, char *argv[] )", "Use Data Folder: " + QDir::currentPath() );
		}
		#else
		
		QDir data_dir;
		
		if( data_dir.exists("/usr/share/aqemu/") )
		{
			settings.setValue( "AQEMU_Data_Folder", "/usr/share/aqemu/" );
			AQDebug( "int main( int argc, char *argv[] )", "Use Data Folder: /usr/share/aqemu/" );
		}
		else if( data_dir.exists("/usr/share/apps/aqemu/") )
		{
			settings.setValue( "AQEMU_Data_Folder", "/usr/share/apps/aqemu/" );
			AQDebug( "int main( int argc, char *argv[] )", "Use Data Folder: /usr/share/apps/aqemu/" );
		}
		else if( data_dir.exists("/usr/local/share/aqemu/") )
		{
			settings.setValue( "AQEMU_Data_Folder", "/usr/local/share/aqemu/" );
			AQDebug( "int main( int argc, char *argv[] )", "Use Data Folder: /usr/local/share/aqemu/" );
		}
		else
		{
			QMessageBox::information( NULL, QObject::tr("Error!"),
				QObject::tr("Cannot Locate AQEMU Data Folder!\nYou Most Select This Folder in Next Window!"),
				QMessageBox::Ok );
			
			QString aqemu_data_dir = QFileDialog::getExistingDirectory( NULL,
				QObject::tr("Please Select AQEMU Data Folder:"), "/",
				QFileDialog::ShowDirsOnly );
			
			if( aqemu_data_dir.isEmpty() )
			{
				QMessageBox::critical( NULL,
					QObject::tr("Error!"), QObject::tr("AQEMU Don't Work If Data Folder Not Selected!") );
				return -1;
			}
			else
			{
				settings.setValue( "AQEMU_Data_Folder", aqemu_data_dir );
			}
		}
		
		#endif
	}
	
	// This is a first start AQEMU on this computer?
	settings.setValue( "Show_Language_Warning", "no" );
	
	if( settings.value("First_Start", "yes").toString() == "yes" )
	{
		QMessageBox::information( NULL, QObject::tr("AQEMU Configuration"),
			QObject::tr("This is a First Start AQEMU You Must Adjust the Program for Work") );
		
		Settings_Window *Settings_Win = new Settings_Window( NULL );
	
		for( int uf = 0; uf < 2 ; ++uf ) // user click OK button
		{
			Settings_Win->Advanced_Win_Show = true;
			
			if( Settings_Win->exec() == QDialog::Accepted )
			{
				settings.setValue( "First_Start", "no" );
				break;
			}
		}
		
		delete Settings_Win;
	}
	
	settings.setValue( "Show_Language_Warning", "yes" );
	
	// Language
	QTranslator appTranslator;
	
	appTranslator.load( settings.value("AQEMU_Data_Folder", "").toString() +
			"aqemu_" + settings.value( "Language", "" ).toString() + ".qm",
						app.applicationDirPath() );
	
	app.installTranslator( &appTranslator );
	
	Main_Window Window;
	Window.show();
	
	return app.exec();
}
